Package: aplpack
Title: Another Plot Package: 'Bagplots', 'Iconplots', 'Summaryplots',
        Slider Functions and Others
Version: 1.3.3
Date: 2019-07-26
Author: Hans Peter Wolf [aut, cre]
Maintainer: Hans Peter Wolf <pwolf@wiwi.uni-bielefeld.de>
Depends: R (>= 3.0.0)
Suggests: tkrplot, jpeg, png, splines, utils, tcltk
Description: Some functions for drawing some special plots:
   The function 'bagplot' plots a bagplot,
   'faces' plots chernoff faces,
   'iconplot' plots a representation of a frequency table or a data matrix,
   'plothulls' plots hulls of a bivariate data set,
   'plotsummary' plots a graphical summary of a data set,
   'puticon' adds icons to a plot,
   'skyline.hist' combines several histograms of a one dimensional data set in one plot,
   'slider' functions supports some interactive graphics,
   'spin3R' helps an inspection of a 3-dim point cloud,
   'stem.leaf' plots a stem and leaf plot,
   'stem.leaf.backback' plots back-to-back versions of stem and leaf plot.
License: GPL (>= 2)
URL:
        http://www.wiwi.uni-bielefeld.de/lehrbereiche/statoekoinf/comet/wolf/wolf_aplpack
NeedsCompilation: no
Packaged: 2019-07-26 14:02:15 UTC; pwolf
Repository: CRAN
Date/Publication: 2019-07-29 07:50:12 UTC
